//=============================================================================
// AM_Pickup.
//
// Author: Francesco Biscazzo
// Date: 2016
//=============================================================================
class AM_Pickup extends TournamentPickup;

var AdvancedMutator AM;
var Weapon OwnerWeapon;

var ZoneInfo oldOwnerZone;
var String oldOwnerState;
var String oldOwnerWeaponState;
var EPhysics oldOwnerPhysics;

function RegisterAdvancedMutator(AdvancedMutator AdvMut) {
	AM = AdvMut;
	SetCollision(true, false, false);
	bCollideWorld = true;
	SetCollisionSize( Owner.CollisionRadius, Owner.CollisionHeight );
	
	GotoState('Activated');
	Activate();
	
	SetTimer(AM.callbackRate, True);
}

function Activate() {}


state Activated
{
	function Activate() {
		bActive = true;
		SetOwnerWeapon();
	}

	function EndState() {
		bActive = false;
		GotoState('Idle2');
	}

	function BeginState() {
		bActive = true;
		SetOwnerWeapon();	
	}
	
	// Use getStateName() on the weapon to get the fire type.
	function FireEffect() {
		AM.Fired(Pawn(Owner), OwnerWeapon, AM.static.getAimedActor(Pawn(Owner), OwnerWeapon));
	}

	function OwnerJumped() {
		super.OwnerJumped();
		
		AM.Jumped(Pawn(Owner));
	}

	function ChangedWeapon() {
		local Weapon OldWeapon;

		OldWeapon = OwnerWeapon;
		super.ChangedWeapon();
		SetOwnerWeapon();
		
		AM.ChangedWeapon(Pawn(Owner), OldWeapon, OwnerWeapon);
	}

	function bool HandlePickupQuery( Inventory item ) {
		AM.TouchedPickUp(Pawn(Owner), item);
		
		return Inventory.HandlePickupQuery(item);
	}
	
	function ZoneChange( ZoneInfo newZone ) {
		AM.ChangedZone(Pawn(Owner), oldOwnerZone, newZone);
		oldOwnerZone = newZone;
	}

	function SetOwnerWeapon() {
		if ( !bActive )
			return;

		// Remove affector from the previous selected weapon.
		if ( OwnerWeapon != None ) {
			OwnerWeapon.SetDefaultDisplayProperties();
			if ( OwnerWeapon.IsA('TournamentWeapon') )
				TournamentWeapon(OwnerWeapon).Affector = None;
		}
		
		// Set new weapon as the owner weapon.
		OwnerWeapon = Pawn(Owner).Weapon;
		
		// Add affector to the owner weapon.
		if ( OwnerWeapon != None ) {
			if ( OwnerWeapon.IsA('TournamentWeapon') )
				TournamentWeapon(OwnerWeapon).Affector = self;
		}
	}
	
	function HitWall( vector HitNormal, actor HitWall ) {
		AM.HittingWall(Pawn(Owner), HitNormal, HitWall);
	}
	
	function Falling() {
		AM.Fell(Pawn(Owner));
	}
	
	function Landed( vector HitNormal ) {
		AM.Landing(Pawn(Owner), HitNormal);
	}
	
	function Touch( Actor Other ) {
		AM.Touching(Pawn(Owner), Other);
	}
	
	function PostTouch( Actor Other ) {
		AM.Touched(Pawn(Owner), Other);
	}
	
	function UnTouch( Actor Other ) {
		AM.UnTouched(Pawn(Owner), Other);
	}
	
	function Bump( Actor Other ) {
		AM.Bumped(Pawn(Owner), Other);
	}
	
	function BaseChange() {
		AM.ChangedBase(Pawn(Owner));
	}
	
	function EncroachedBy( actor Other ) {
		AM.Encroached(Pawn(Owner), Other);
	}
	
	function EndedRotation() {
		AM.EndingRotation(Pawn(Owner));
	}

	function FellOutOfWorld() {
		AM.FallingOutOfWorld(Pawn(Owner));
	}

	function Timer() {
		local String currentOwnerState;
		local String currentOwnerWeaponState;
		local EPhysics currentOwnerPhysics;
	
		if (o()) SetLocation(Owner.Location);
		if (o()) SetRotation(Owner.Rotation);
		
		if (o()) if (Owner.IsA('Pawn'))
				if (o()) Pawn(Owner).bCountJumps = true;

		if (o()) if ( !Pawn(Owner).bAutoActivate )
				OwnerJumped();
	
		if (o()) currentOwnerState = String(Owner.getStateName());
		if (currentOwnerState != oldOwnerState) {
			if (o()) AM.ChangedState(Pawn(Owner), oldOwnerState, currentOwnerState);
			oldOwnerState = currentOwnerState;
		}
		
		if (w()) currentOwnerWeaponState = String(OwnerWeapon.getStateName());
		if (currentOwnerWeaponState != oldOwnerWeaponState) {
			if (o()) AM.ChangedWeaponState(Pawn(Owner), OwnerWeapon, oldOwnerWeaponState, currentOwnerWeaponState);
			
			oldOwnerWeaponState = currentOwnerWeaponState;
		}
		
		if (o()) currentOwnerPhysics = Owner.Physics; else currentOwnerPhysics = PHYS_None;
		if (currentOwnerPhysics != oldOwnerPhysics) {
			if (o()) AM.ChangedPhysics(Pawn(Owner), oldOwnerPhysics, currentOwnerPhysics);
			oldOwnerPhysics = currentOwnerPhysics;
			
			SetPhysics(currentOwnerPhysics);
		}
		
		if (o()) AM.AimedActor(Pawn(Owner), OwnerWeapon, AM.static.getAimedActor(Pawn(Owner), OwnerWeapon));
		
		AM.EveryTimeCallback(self);
	}
}

auto state Pickup {
Begin:
	GotoState('Idle');
}

function bool o() {
	return Owner != none;
}

function bool w() {
	return OwnerWeapon != none;
} 