//=============================================================================
// AdvancedMutator.
//
// Author: Francesco Biscazzo
// Date: 2016
//=============================================================================
class AdvancedMutator extends Mutator;

var() bool bAutoRegister; // Register this mutator as next mutator of the GameInfo.BaseMutator.
var() float callbackRate;

function PostBeginPlay() {
	super.PostBeginPlay();
	
	if (bAutoRegister)
		Level.Game.BaseMutator.NextMutator = self;
	
	if (callbackRate <= 0)
		callbackRate = 0.01;
}

function ModifyPlayer(Pawn Other) {
	local AM_Pickup AMP;

	AMP = Spawn(class'AM_Pickup', Other);
	if (AMP != none) {
		AMP.bHeldItem = true;
		AMP.GiveTo( Other );
		
		AMP.RegisterAdvancedMutator(self);
	}
	
	super.ModifyPlayer(Other);
}

function AM_Pickup getPawnAMP(Pawn pawn) {
	local AM_Pickup AMP;
	
	foreach AllActors(class'AM_Pickup', AMP) {
		if (AMP.Owner == pawn)
			return AMP;
	}
	
	return none;
}

function Fired(Pawn pawn, Weapon weapon, Actor HitActor);

function Jumped(Pawn pawn);

function ChangedWeapon(Pawn pawn, Weapon oldWeapon, Weapon newWeapon);

function TouchedPickUp(Pawn pawn, Inventory item);

function ChangedZone(Pawn pawn, ZoneInfo oldZone, ZoneInfo newZone);

function ChangedState(Pawn pawn, String oldState, String newState);

function ChangedWeaponState(Pawn pawn, Weapon weapon, String oldState, String newState);

function ChangedPhysics(Pawn pawn, EPhysics oldPhysics, EPhysics newPhysics);

function AimedActor(Pawn pawn, Weapon weapon, Actor aimedActor);

function EveryTimeCallback(AM_Pickup AMP);

function HittingWall( Pawn pawn, vector HitNormal, actor HitWall ) {
	Level.Game.BroadcastMessage("HittingWall"@HitNormal@HitWall);
}

function Fell(Pawn pawn) {
	Level.Game.BroadcastMessage("fell");
}

function Landing( Pawn pawn, vector HitNormal );

function Touching( Pawn pawn, Actor Other );

function Touched( Pawn pawn, Actor Other );

function UnTouched( Pawn pawn, Actor Other );

function Bumped( Pawn pawn, Actor Other ) {
	Level.Game.BroadcastMessage("Bumped"@Other);
}

function ChangedBase(Pawn pawn);

function Encroached( Pawn pawn, actor Other );

function EndingRotation(Pawn pawn) {
	Level.Game.BroadcastMessage("EndingRotation");
}

function FallingOutOfWorld(Pawn pawn) {
	Level.Game.BroadcastMessage("FallingOutOfWorld");
}

static function actor getAimedActor(Pawn pawn, Weapon weapon) {
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local float Accuracy;
	
	local rotator AdjustedAim; //To not to affect the pawn.Weapon.AdjustedAim
	
	if ((pawn != none) && (weapon != none)) {
		Accuracy = 0.0;

		GetAxes(pawn.ViewRotation, X, Y, Z);
		StartTrace = pawn.Location + weapon.CalcDrawOffset() + weapon.FireOffset.X * X + weapon.FireOffset.Y * Y + weapon.FireOffset.Z * Z; 
		AdjustedAim = pawn.AdjustAim(1000000, StartTrace, 2*weapon.AimError, False, False);	
		EndTrace = StartTrace + Accuracy * (weapon.FRand() - 0.5 )* Y * 1000
			+ Accuracy * (weapon.FRand() - 0.5 ) * Z * 1000;
		X = vector(AdjustedAim);
		EndTrace += (10000 * X);
		
		return pawn.TraceShot(HitLocation, HitNormal, EndTrace, StartTrace);
	}
	
	return none;
}

defaultproperties {
	callbackRate=0.01
}